/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * ****/

import { SColor, SFont } from "@persagy-web/draw";
import { imgBaseUrl } from "@/api/imageservice";
import { EquipMap, SPlanEquipment, SPlanZone, Style, StyleMap, ZoneMap } from ".";

/**
 * 平面图装饰器
 *
 *  @author  张宇 <taohuzy@163.com>
 */
export class SPlanDecorator {
    /**
     * 设备元素装饰器
     *
     * @param equipItem    设备元素
     * @param style    样式数据
     */
    static equipDecorator(equipItem: SPlanEquipment, style: Style): void {
        // 设置设备字体颜色
        if (style.color) equipItem.color = new SColor(style.color);
        // 设置设备字体大小
        if (style.size) equipItem.font = new SFont("sans-serif", style.size);
        // 设置图片地址
        if (style.url) {
            equipItem.url = `${imgBaseUrl}${style.url}`;
            equipItem.defaultUrl = style.url;
        }
        // 设置设备公式
        if (style.formula) equipItem.formula = style.formula;
        // 设置设备图标宽度
        if (style.width) equipItem.sWidth = style.width;
        // 设置设备图标高度
        if (style.height) equipItem.sHeight = style.height;
    }

    /**
     * 空间元素装饰器
     *
     * @param zoneItem 空间元素
     * @param style 样式数据
     */
    static zoneDecorator(zoneItem: SPlanZone, style: Style): void {
        // 设置空间字体颜色
        if (style.color) zoneItem.color = new SColor(style.color);
        // 设置空间字体大小
        if (style.size) zoneItem.font = new SFont("sans-serif", style.size);
        // 设置空间填充色
        if (style.fillColor) zoneItem.fillColor = new SColor(style.fillColor);
        // 设置空间边框颜色
        if (style.strokeColor) zoneItem.strokeColor = new SColor(style.strokeColor);
        // 设置空间边框线宽
        if (style.lineWidth) zoneItem.lineWidth = style.lineWidth;
        // 设置空间边框线型
        if (style?.lineStyle && style.lineStyle >= 0) zoneItem.lineStyle = style.lineStyle;
        // 设置设备公式
        if (style.formula) zoneItem.formula = style.formula;
    }

    /**
     * 设备类装饰器
     *
     * @param classCode 设备类型编码
     * @param equipItemMap 设备图元素库
     * @param styleMap 样式库
     */
    static equipMapDecorator(classCode: string, equipItemMap: EquipMap, styleMap: StyleMap): void {
        Object.values(equipItemMap).forEach((equipItem: SPlanEquipment) => {
            // 设备类型相同且该设备没独立设置过样式，则该设备应用设备类型样式
            if (equipItem.legendData?.classCode === classCode && !styleMap[equipItem.id] && styleMap[classCode])
                this.equipDecorator(equipItem, styleMap[classCode]);
        });
    }

    /**
     * 空间类装饰器
     *
     * @param classCode 空间类型编码
     * @param zoneItemMap 空间图元素库
     * @param styleMap 样式库
     */
    static zoneMapDecorator(classCode: string, zoneItemMap: ZoneMap, styleMap: StyleMap): void {
        Object.values(zoneItemMap).forEach((zoneItem: SPlanZone) => {
            // 设备类型相同且该设备没独立设置过样式，则该设备应用设备类型样式
            if (zoneItem.legendData?.classCode === classCode && !styleMap[zoneItem.id] && styleMap[classCode])
                this.zoneDecorator(zoneItem, styleMap[classCode]);
        });
    }
}
