/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * ****/

import { SParser } from "@persagy-web/big/lib";
import { BigEditFactory, Legend, Marker, Relation, ElementData } from "@persagy-web/big-edit";

/**
 * 平面图解析器
 *
 *  @author  张宇 <taohuzy@163.com>
 */
export class SPlanParser extends SParser {
    /**图例节点 */
    nodes: any = []; // 图例节点，所有与工程信息化相关的图例（图标类型与区域） // 与工程信息无关的标识对象（增加文本注释，图上的图片说明）
    /**图例节点 */
    markers: any = [];
    /**  管线对象 */
    relations: any = [];
    constructor() {
        super(new BigEditFactory());
    }

    /**
     * 解析拓扑图绘制数据
     *
     * @param data    业务空间数据
     */
    parseData(data: ElementData): void {
        // 生成遍历基本图例
        if (data.markers && data.markers.length) {
            data.markers.forEach((item) => {
                this.addMarker(item);
            });
        }

        // 生成遍历Node图例
        if (data.nodes && data.nodes.length) {
            data.nodes.forEach((item: any) => {
                this.addNode(item);
            });
        }

        // 生成遍历关系图例
        if (data.relations && data.relations.length) {
            data.relations.forEach((item: any) => {
                this.addRelation(item);
            });
        }
    }

    /**
     * 添加生成 mark 实例
     *
     * @param data Marker   图例类型数据
     */
    addMarker(data: Marker) {
        if (data.properties && data.properties.type) {
            switch (data.properties.type) {
                case "BaseLine":
                    this.markers.push(this.factory.createBaseLineEdit(data));
                    break;
                case "BaseText":
                    this.markers.push(this.factory.createBaseTextEdit(data));
                    break;
                case "BaseImage":
                    this.markers.push(this.factory.createBaseImageEdit(data));
                    break;
                case "BaseExplain":
                    this.markers.push(this.factory.createBaseExpainEdit(data));
                    break;
                case "BaseCircle":
                    this.markers.push(this.factory.createBaseCircleEdit(data));
                    break;
                case "BaseArrow":
                    this.markers.push(this.factory.createBaseArrow(data));
                    break;
                case "BaseRect":
                    this.markers.push(this.factory.createBaseRectEdit(data));
                    break;
                case "BasePolygon":
                    this.markers.push(this.factory.createBasePolygonEdit(data));
                    break;
                case "BaseArrowPolygon":
                    this.markers.push(this.factory.createBasePolygonArrowEdit(data));
            }
        }
    }

    /**
     * 添加生成 Node 实例
     *
     * @param data Legend   图例类型数据
     */
    addNode(data: Legend) {
        let node = null;
        if (data.properties && data.properties.type) {
            switch (data.properties.type) {
                case "BaseEquipment":
                    node = this.factory.createBaseEquipment(data);
                    break;
            }
        }
        if (node) {
            this.nodes.push(node);
        }

        return node;
    }

    /**
     * 添加生成 Relation 实例
     *
     * @param data Relation   图例类型数据
     */
    addRelation(data: Relation) {
        let relation = null;
        if (data.properties && data.properties.type) {
            switch (data.properties.type) {
                case "BasePipe":
                    relation = this.factory.createBasePipe(data);
                    break;
            }
        }

        if (relation) {
            this.relations.push(relation);
        }

        return relation;
    }
}
