/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2016-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SMouseEvent } from "@persagy-web/base";
import { SGraphItem, SLineStyle, STextOrigin, Point } from "@persagy-web/graph";
import { SColor, SFont, SPainter, SLineCapStyle, SPoint, SPath, SPolygonUtil, SRect } from "@persagy-web/draw";
import { Example } from "../index";
import { SGraphEdit, SBaseTextEdit } from "@persagy-web/edit";
import { SItemStatus } from "@persagy-web/big";
/**
 * 编辑基础空间类
 *
 * @author 张宇 <taohuzy@163.com>
 */
export class SPlanZone extends SGraphEdit {
    /** X 坐标最小值 */
    private minX = Number.MAX_SAFE_INTEGER;
    /** X 坐标最大值 */
    private maxX = Number.MIN_SAFE_INTEGER;
    /** Y 坐标最小值 */
    private minY = Number.MAX_SAFE_INTEGER;
    /** Y 坐标最大值 */
    private maxY = Number.MIN_SAFE_INTEGER;
    /** 墙轮廓线坐标 list */
    private pointList: SPoint[][][] = [];
    /** path 对象 */
    private pathList: SPath[] = [];
    /** 文本数组 */
    textItemList: SBaseTextEdit[] = [];

    /** 是否可移动 */
    set moveable(val: boolean) {
        return;
    }
    /** 设备图例 */
    private _legendData: Example | null = null;
    set legendData(val) {
        this._legendData = val;
        this.initData();
    } // set legendData()
    get legendData(): Example | null {
        return this._legendData;
    } // get legendData()

    /** item 状态 */
    _status: SItemStatus = SItemStatus.Normal;
    get status(): SItemStatus {
        return this._status;
    }
    set status(v: SItemStatus) {
        this._status = v;
        if (v == SItemStatus.Normal) {
            this.textItemList.map((item) => {
                item.moveable = false;
            });
        } else if (v == SItemStatus.Edit) {
            this.textItemList.map((item) => {
                item.moveable = true;
            });
        }
        this.update();
    }

    /** 边框颜色 */
    _strokeColor: SColor = SColor.Black;
    get strokeColor(): SColor {
        return this._strokeColor;
    } // Get strokeColor
    set strokeColor(v: SColor) {
        this._strokeColor = new SColor(v);
        this.update();
    } // Set strokeColor

    /** 填充颜色 */
    _fillColor: SColor = SColor.Black;
    get fillColor(): SColor {
        return this._fillColor;
    } // Get fillColor
    set fillColor(v: SColor) {
        this._fillColor = new SColor(v);
        this.update();
    } // Set fillColor

    /** 线条样式 */
    _lineStyle: SLineStyle = SLineStyle.Solid;
    get lineStyle(): SLineStyle {
        return this._lineStyle;
    } // Get lineStyle
    set lineStyle(v: SLineStyle) {
        this._lineStyle = v;
        this.update();
    } // Set lineStyle

    /** 线条宽度 */
    _lineWidth: number = 1;
    get lineWidth(): number {
        return this._lineWidth;
    } // Get lineWidth
    set lineWidth(v: number) {
        this._lineWidth = v;
        this.update();
    } // Set lineWidth

    /** 宽度是否像素 */
    widthIsPx: boolean = false;

    /** 文本颜色 */
    private _color: SColor = new SColor();
    get color(): SColor {
        return this._color;
    } // Get color
    set color(v: SColor) {
        this._color = v;
        this.textItemList.forEach((item) => {
            item.color = v;
        });
        this.update();
    }

    /** 文本字体 */
    private _font: SFont = new SFont("sans-serif", 12);
    get font(): SFont {
        return this._font;
    } // Get font
    set font(v: SFont) {
        this._font = v;
        this.textItemList.forEach((item: SBaseTextEdit, index: number) => {
            item.font = v;
        });
        this.update();
    } // Set font

    /** 是否显示文字 */
    _showText: boolean = true;
    get showText(): boolean {
        return this._showText;
    }
    set showText(v: boolean) {
        if (v === this._showText) {
            return;
        }
        this._showText = v;
        if (v) {
            this.textItemList.map((item) => {
                item.show();
            });
        } else {
            this.textItemList.map((item) => {
                item.hide();
            });
        }
    }

    /** 公式 */
    private _formula: any[] = [];
    set formula(val) {
        this._formula = val;
        try {
            if (this.scene) {
                this.textItemList.forEach((textItem) => {
                    this.scene?.removeItem(textItem);
                });
            }
            if (val.length) {
                const textItemList: any[] = [];
                val.forEach((item: any, index: number) => {
                    let obj = new SBaseTextEdit(this, null);
                    obj.propertyData = { ...item };
                    obj.text = item.name;
                    if (item.pos) {
                        obj.moveTo(item.pos.x, item.pos.y);
                    } else {
                        const avgPos = this.getAverageVal();
                        obj.moveTo(avgPos.avgX, avgPos.avgY - this.font.size * (0.5 * val.length - 0.5 - index));
                    }

                    obj.connect("onMove", this, this.textMove.bind(this));
                    obj.font = this.font;
                    obj.color = this.color;
                    obj.originPosition = STextOrigin.Center;
                    obj.isTransform = false;
                    obj.showSelect = false;
                    textItemList.push(obj);
                });
                this.textItemList = textItemList;
            } else {
                this.textItemList = [];
            }
            this.update();
        } catch (error) {
            console.error("公式数据错误", error);
        }
    } // set formula()
    // get formula(): string {
    get formula(): any[] {
        return this._formula;
    } // get formula()

    //  设备附加数据
    anotherMsg: string = "";

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data      数据
     */
    constructor(parent: SGraphItem | null, data: Example) {
        super(parent);
        this.zOrder = 9700;
        this.selectable = true;
        this.showSelect = false;
        this.legendData = data;
    } // Constructor

    /**
     * 设置 legendData 时对 item 做设置
     */
    initData() {
        if (!this.legendData) return;
        this.id = this.legendData.id ? this.legendData.id : "";
        this.anotherMsg = this.legendData?.properties?.anotherMsg ? this.legendData.properties.anotherMsg : "";
        if (this.legendData?.outline?.length && this.legendData.outline[0] && this.legendData.outline[0]?.length) {
            let outline = this.legendData.outline.map((t): Point[][] => {
                return t.map((it): Point[] => {
                    return it.map(
                        (item): Point => {
                            return { x: item.X, y: -item.Y };
                        }
                    );
                });
            });
            this.minX = outline[0][0][0].x;
            this.maxX = this.minX;
            this.minY = outline[0][0][0].y;
            this.maxY = this.minY;
            // 处理轮廓点数组,同时计算最大最小值
            this.pointList = outline.map((t): SPoint[][] => {
                let sPath = new SPath();
                let tempArr = t.map((it): SPoint[] => {
                    let array = it.map(
                        (item): SPoint => {
                            let x = item.x,
                                y = item.y;
                            if (x < this.minX) {
                                this.minX = x;
                            }
                            if (y < this.minY) {
                                this.minY = y;
                            }
                            if (x > this.maxX) {
                                this.maxX = x;
                            }
                            if (y > this.maxY) {
                                this.maxY = y;
                            }
                            return new SPoint(x, y);
                        }
                    );
                    sPath.polygon(array);
                    return array;
                });
                this.pathList.push(sPath);
                return tempArr;
            });
        } else {
            this.minX = 0;
            this.maxX = 0;
            this.minY = 0;
            this.maxY = 0;
        }
    }

    /**
     * 获取轮廓线第一块区域的坐标平均值
     */
    getAverageVal() {
        let avgX = 0,
            avgY = 0;
        if (this.pointList[0] && this.pointList[0][0] && this.pointList[0][0].length) {
            const firstOutline = this.pointList[0][0];
            avgX =
                firstOutline.reduce((val, pos) => {
                    return val + pos.x;
                }, 0) / firstOutline.length;
            avgY =
                firstOutline.reduce((val, pos) => {
                    return val + pos.y;
                }, 0) / firstOutline.length;
        }
        return { avgX, avgY };
    }

    /**
     * 文本位置移动回调函数
     *
     * @param textItem 移动的文本元素
     * @param pointList 移动前后位置坐标信息
     */
    textMove(textItem: SBaseTextEdit, pointList: SPoint[]) {
        this.moveText();
    }
    moveText() {}

    /**
     * 鼠标双击事件
     *
     * @param event   鼠标事件
     * @return 是否处理事件
    */
   onDoubleClick(event: SMouseEvent): boolean {
    // 如果位show状态 双击改对象则需改为编辑状态
    if (SItemStatus.Normal == this.status) {
        this.status = SItemStatus.Edit;
        this.grabItem(this);
    } else if (SItemStatus.Edit == this.status) {
        this.status = SItemStatus.Normal;
        this.releaseItem();
    }

    this.update();
    return true;
}

    /**
     * Item 对象边界区域
     *
     * @return 对象边界区域
     */
    boundingRect(): SRect {
        let rect = new SRect(this.minX, this.minY, this.maxX - this.minX, this.maxY - this.minY);
        if (this.showText) {
            this.textItemList.forEach(item => {
                rect = rect.unioned(
                    item
                        .boundingRect()
                        .adjusted(item.pos.x, item.pos.y, 0, 0)
                );
            })
        }
        return rect.adjusted(-5, -5, 10, 10);
    }

    /**
     * 判断点是否在区域内
     *
     * @param x 点 x 坐标
     * @param y 点 y 坐标
     * @return 是否在区域内
     */
    contains(x: number, y: number): boolean {
        for (let j = 0; j < this.pointList.length; j++) {
            let arr = this.pointList[j];
            if (arr.length < 1 || !SPolygonUtil.pointIn(x, y, arr[0])) {
                continue;
            }
            if (arr.length == 1) {
                return true;
            }
            let flag = false;
            for (let i = 1; i < arr.length; i++) {
                if (SPolygonUtil.pointIn(x, y, arr[i])) {
                    flag = true;
                    break;
                }
            }
            if (flag) {
                continue;
            }
            return true;
        }
        return false;
    }

    /**
     * 返回对象储存的相关数据
     *
     * @return	对象储存的相关数据
     */
    toData(): any {
        if (this.legendData) {
            // 位置
            const objExtInfo = {
                attachObjectIds: [this.id],
                properties: {
                    id: this.id,
                    anotherMsg: this.anotherMsg,
                },
            };
            return objExtInfo;
        }
        return new Object();
    }

    /**
     * Item 绘制操作
     *
     * @param painter    绘制对象
     */
    onDraw(painter: SPainter) {
        const rect = this.boundingRect();
        const lw = painter.toPx(1);
        // 编辑态出现四个角的圆点和外接矩阵轮廓
        if (this.status == SItemStatus.Edit) {
            painter.pen.lineWidth = lw;
            painter.pen.lineDash = [3 * lw, 7 * lw];
            painter.pen.color = SColor.Black;
            painter.brush.color = SColor.Transparent;
            painter.drawRect(rect);

            painter.pen.lineDash = [];
            painter.brush.color = SColor.White;
            painter.drawCircle(rect.x, rect.y, 5 * lw)
            painter.drawCircle(rect.right, rect.bottom, 5 * lw)
            painter.drawCircle(rect.x, rect.bottom, 5 * lw)
            painter.drawCircle(rect.right, rect.y, 5 * lw)
        }

        painter.pen.lineCapStyle = SLineCapStyle.Square;
        painter.pen.color = this.strokeColor;
        painter.brush.color = this.fillColor;
        painter.pen.lineWidth = painter.toPx(this.lineWidth);
        if (this.lineStyle == SLineStyle.Dashed) {
            painter.pen.lineDash = [painter.toPx(this.lineWidth * 3), painter.toPx(this.lineWidth * 7)];
        } else if (this.lineStyle == SLineStyle.Dotted) {
            painter.pen.lineDash = [painter.toPx(this.lineWidth), painter.toPx(this.lineWidth)];
        }

        // 是否选中
        if (this.selected) {
            painter.shadow.shadowBlur = 10;
            painter.shadow.shadowColor = new SColor(`#00000033`);
            painter.shadow.shadowOffsetX = 5;
            painter.shadow.shadowOffsetY = 5;
        } else {
            painter.shadow.shadowColor = SColor.Transparent;
        }

        this.pathList.forEach((t): void => {
            painter.drawPath(t);
        });
    }
}
