/**
 *
 * @param {Array} arr 转换成树的数组
 * @param {String} parentId 父id字段名
 * @param {String} id id名称,默认为"id"
 */
function array2Tree(arr, parentId, id = "id") {
    if (!Array.isArray(arr)) throw new Error("请传入数组");
    if (!parentId || typeof parentId !== "string") throw new Error("请传入 父id 字段名称");
    let map = {};
    arr.map((item) => (map[item[id]] = item));
    let roots = [];
    arr.map((item) => {
        const parent = map[item[parentId]];
        if (parent) {
            (parent.children || (parent.children = [])).push(item);
        } else {
            roots.push(item);
        }
    });
    map = null;
    return roots;
}
// const person = {
//     name: "sudada",
//     gender: "female",
// };
// const omit = (prop, { [prop]: _, ...rest }) => rest;
// const newObj = omit("name", person); // 删除 name 属性

/**
 *
 * @param {*} tree 需要转换的tree,可以是树,或者是数组,每一项是树
 * @param  {...any} transfor  转换的键名  'id->_id' 老键名->新键名    mapTree(tree, "id->_id", "code->id")
 */
function mapTree(tree, ...transfor) {
    //  传入数组
    const arrayTree = Array.isArray(tree);
    if (arrayTree) {
        return tree.map((i) => mapTree(i, ...transfor));
    } else {
        // 传入对象
        const haveChildren = Array.isArray(tree.children) && tree.children.length;
        const obj = {};
        transfor.map((v) => {
            const key = v.split("->")[0],
                newKey = v.split("->")[1];
            obj[newKey] = tree[key];
        });
        return {
            //分别将我们查询出来的值做出改变他的key
            ...tree,
            ...obj,
            // _id: tree.id,
            // id: tree.code,
            //判断它是否存在子集，若果存在就进行再次进行遍历操作，知道不存在子集便对其他的元素进行操作
            children: haveChildren ? tree.children.map((i) => mapTree(i, ...transfor)) : [],
        };
    }
}
/**
 * 
 * @param {*} dataURI 
 * @param {*} type 
 */
function dataURLtoBlob(dataURI, type) {
    const binary = atob(dataURI.split(",")[1]);
    const array = [];
    for (let i = 0; i < binary.length; i++) {
        array.push(binary.charCodeAt(i));
    }
    return new Blob([new Uint8Array(array)], { type: type });
}
/**
 * 格式化时间
 * @param {Date} now 需要格式化的时间
 */
function formatDate(now) {
    const year = now.getFullYear();
    const month = now.getMonth() + 1;
    const date = now.getDate();
    const hour = now.getHours();
    const minute = now.getMinutes();
    const second = now.getSeconds();
    return (
        year +
        "-" +
        month +
        "-" +
        (date > 10 ? date : "0" + date) +
        " " +
        hour +
        ":" +
        (minute > 10 ? minute : "0" + minute) +
        ":" +
        (second > 10 ? second : "0" + second)
    );
}
export { array2Tree, mapTree, dataURLtoBlob, formatDate };
